<?php
/*
Plugin Name: Pushover Notifications for WordPress
Plugin URI: http://www.chriskdesigns.com/plugins/pushover-notifications
Description: Pushover Notifications allows your WordPress blog to send push notifications for events happening on your blog straight to your iOS device with the Pushover app. This plugin is not associated with the Pushover Notifications team or Superblock.
Version: 1.7
Author: Chris Klosowski
Author URI: http://www.chriskdesigns.com/
License: GPLv2 or later
*/

define( 'CKPN_CORE_TEXT_DOMAIN', 'ckpn' );

class CKPushoverNotifications {
	private static $ckpn_instance;

	private function __construct() {
		$options = $this->getOptions();

		add_action( 'init', array( $this, 'ckpn_loaddomain', ) );

		if ( $options['new_user'] ) {
			add_action( 'user_register', array( $this, 'ckpn_user_registration' ) );
		}
		if ( $options['new_comment'] ) {
			add_action( 'comment_post', array( $this, 'ckpn_new_comment' ) );
		}
		if ( $options['password_reset'] ) {
			add_action( 'lostpassword_post', array( $this, 'ckpn_lost_password_request' ) );
		}
		if ( is_admin() ) {
			add_action( 'admin_notices', array( $this, 'ckpn_edd_missing_nag' ) );
			add_action( 'admin_init', array( $this, 'ckpn_register_settings' ), 1000, 0 );
			add_action( 'admin_menu', array( $this, 'ckpn_setup_admin_menu' ), 1000, 0 );
			add_filter( 'user_contactmethods', array( $this, 'ckpn_add_contact_item' ), 10, 1 );
		}
	}

	public static function getInstance() {
		if ( !self::$ckpn_instance ) {
			self::$ckpn_instance = new CKPushoverNotifications();
		}

		return self::$ckpn_instance;
	}

	private static function getOptions() {
		static $current_options = NULL;

		if ( $current_options !== NULL )
			return $current_options;

		$defaults = array(
				'application_key' 	=> false,
				'api_key'			=> false,
				'new_user'			=> false,
				'new_comment'		=> false,
				'notify_authors'	=> false,
				'password_reset'	=> false,
				'sslverify'			=> false,
				'logging'			=> false
			);

		$defaults = apply_filters( 'ckpn_options_defaults', $defaults );
		
		$options = wp_parse_args( get_option( 'ckpn_pushover_notifications_settings' ), $defaults );

		return $options;
	}

	public function ckpn_send_notification( $passed_args ) {
		$logentry = '';
		$invalid = false;
		$options = $this->getOptions();
		$defaults = array(
			'token' => $options['application_key'],
			'user' => NULL );

		$api_args = wp_parse_args( $passed_args, $defaults );

		// Check that we have the required elements
		if ( !isset( $api_args['title'] ) ) {
			$logentry .= date( 'm-d-Y H:i:s' ) . ' -- ****** Missing Title ******' . "\n";
			$invalid = true;
		}

		if ( !isset( $api_args['message'] ) ) {
			$logentry .= date( 'm-d-Y H:i:s' ) . ' -- ****** Missing Message ******' . "\n";
			$invalid = true;
		}

		if ( $api_args['user'] == NULL ) { // Use the default key if none was provided
			$api_args['user'] = $options['api_key'];
		}

		if ( !isset( $api_args['user'] ) ) {
			$invalid = true;
		}

		if ( !$invalid ) {
			/*
			 * Clean house a bit to meet the guidelines of the api
			 *
			 * https://pushover.net/api#limits
			 */
			// Title + Message Limit: 512 Characters
			if ( strlen( $api_args['message'] ) + strlen( $api_args['title'] ) > 512 ) {
				$total_len = strlen( $api_args['message'] ) + strlen( $api_args['title'] );
				$overage = $total_len - 520;
				$api_args['message'] = substr( $api_args['message'], 0, -$overage );
				$last_space = strrpos( $api_args['message'], ' ' );
				$api_args['message'] = substr( $api_args['message'], 0, $last_space ) . '&hellip;';
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- Title & Message too long, truncating message' . "\n";
			}
			// Let's also get entities ready
			$api_args['title']  = html_entity_decode( $api_args['title'], ENT_QUOTES, 'UTF-8' );
			$api_args['message'] = html_entity_decode( $api_args['message'], ENT_QUOTES, 'UTF-8' );

			// URL limit: 500 Characters
			if ( isset( $api_args['url'] ) && strlen( $api_args['url'] ) > 500 ) {
				unset ( $api_args['url'], $api_args['url_title'] );
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- URL was over 500 characters. Removed URL and URL Title' . "\n";
			}

			// URL Title limit: 50 Characters
			if ( isset( $api_args['url_title'] ) && strlen( $api_args['url_title'] ) > 50 ) {
				unset ( $api_args['url_title'] );
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- URL Title was over 50 characters. Removed URL Title' . "\n";
			}

			if ( isset( $api_args['url'] ) ) {
				$api_args['url'] = urlencode( $api_args['url'] );
			}

			// Crate the request
			$body = '';
			foreach ( $api_args as $key => $value ) {
				$body .= $key . '=' . $value . '&';
			}
			$body = substr( $body, 0, -1 );

			$req_args = array( 'body'=>$body );
			if ( $options['sslverify'] == 0 ) {
				$req_args['sslverify'] = false;
			}
			$response = wp_remote_post( 'https://api.pushover.net/1/messages.json', $req_args );

			if ( is_wp_error( $response ) ) { // If the API Fails for some reason send it via the email system of Pushover
				$to = $user_key.'@api.pushover.net';
				if ( wp_mail( $to, $subject, $message ) ) {
					$logentry .= date( 'm-d-Y H:i:s' ) . ' -- Email Notification - ' . $api_args['user'] . ' - Subject: ' . $api_args['title'] . "\n";
				} else {
					$logentry .= date( 'm-d-Y H:i:s' ) . ' -- ******Email Notification Failed******' . "\n";
				}
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- ******API Notification Failed******' . "\n";
			} elseif ( $response['response']['code'] == 500 ) {
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- API Response - Internal Server Error 500' . "\n";
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- Typically occurs with non-entitied characters' . "\n";
			} else {
				$logentry .= date( 'm-d-Y H:i:s' ) . ' -- API Notification - ' . $api_args['user'] . ' - Subject: ' . $api_args['title'] . "\n";
			}
		}

		if ( $options['logging'] == 1 ) {
			$current_logs = get_option( 'ckpn_logs' );
			$new_logs = $logentry . $current_logs;

			$logs_array = explode( "\n", $new_logs );
			if ( count( $logs_array ) > 100 ) {
				$logs_array = array_slice( $logs_array, 0, 100 );
				$new_logs = implode( "\n", $logs_array );
			}

			update_option( 'ckpn_logs', $new_logs );
		}
	}

	public function ckpn_user_registration( $user_id ) {
		$title = apply_filters( 'ckpn_newuser_subject', get_bloginfo( 'name' ) . ': ' . __( 'A New User Has Registered', CKPN_CORE_TEXT_DOMAIN ) );
		$user_data = get_userdata( $user_id );
		$message = apply_filters( 'ckpn_newuser_message', sprintf( __( '%s created an account.', CKPN_CORE_TEXT_DOMAIN ), $user_data->user_login ) );
		$args = array( 'title' => $title, 'message' => $message );
		$this->ckpn_send_notification( $args );
	}

	public function ckpn_new_comment( $comment_id ) {
		$comment_data = get_comment( $comment_id );
		if ( $comment_data->comment_approved == 'spam' ) { // This is not the comment we're looking for. Move Along.
			return;
		}
		if ( $comment_data->comment_type == 'pingback' || $comment_data->comment_type == 'trackback' ) { // They love us, they really love us
			$comment_type = $comment_data->comment_type;
		} else {
			$comment_type = 'comment';
		}

		$title = apply_filters( 'ckpn_newcomment_subject', get_bloginfo( 'name' ) . ': ' . sprintf( __( 'A New %s has been submitted', CKPN_CORE_TEXT_DOMAIN ), $comment_type ) );
		$post_data = get_post( $comment_data->comment_post_ID );
		$message = apply_filters( 'ckpn_newcomment_message', sprintf( __( 'by %1$s on %2$s', CKPN_CORE_TEXT_DOMAIN ), $comment_data->comment_author, $post_data->post_title ) );

		// Notify the Admin User
		$args = array( 'title' => $title, 'message' => $message );
		
		$this->ckpn_send_notification( $args );

		// Check if we should notify the author as well
		$options = $this->getOptions();
		if ( $options['notify_authors'] ) {
			$author_user_key = get_user_meta( $post_data->post_author, 'ckpn_user_key', true );
			if ( $author_user_key != '' && $author_user_key != $options['api_key'] ) { // Only send if the user has a key and it's not the same as the admin key
				// Notify the Author their post has a comment
				$args['user'] = $author_user_key;
				$this->ckpn_send_notification( $args );
			}
		}
	}

	public function ckpn_lost_password_request() {
		if ( !empty( $_POST['user_login'] ) ) {
			if ( strpos( $_POST['user_login'], '@' ) ) {
				$user_data = get_user_by( 'email', trim( $_POST['user_login'] ) );
			} else {
				$login = trim( $_POST['user_login'] );
				$user_data = get_user_by( 'login', $login );
			}
		}

		if ( !empty( $user_data ) ) {
			$user_pushover_key = get_user_meta( $user_data->data->ID, 'ckpn_user_key', true );
			if ( $user_pushover_key != '' ) {
				$title = apply_filters( 'ckpn_password_request_subject', get_bloginfo( 'name' ) . ': ' . __( 'Password Reset Request', CKPN_CORE_TEXT_DOMAIN ) );
				$message = apply_filters( 'ckpn_password_request_message', sprintf( __( 'Someone requested to reset the password for your account, %s. If this was not you pelase verify your account is secure.', CKPN_CORE_TEXT_DOMAIN ), $user_data->data->user_login ) );

				$args = array( 'title' => $title, 'message' => $message, 'user' => $user_pushover_key, 'priority' => 1 );
				$this->ckpn_send_notification( $args );
			}
		}
	}

	public function ckpn_setup_admin_menu() {
		add_options_page( __( 'Pushover Notifications', CKPN_CORE_TEXT_DOMAIN ), __( 'Pushover Notifications', CKPN_CORE_TEXT_DOMAIN ), 'administrator', 'pushover-notifications', array( $this, 'ckpn_admin_page' ) );
	}

	public function ckpn_admin_page() {
		if ( isset( $_GET['clear_logs'] ) && $_GET['clear_logs'] == 'true' ) {
			check_admin_referer( 'ckpn_clear_logs' );
			update_option( 'ckpn_logs', '' );
			$logs_cleared = true;
		}
		$current = $this->getOptions();
		?>
		<div class="wrap">
			<div id="icon-options-general" class="icon32"></div><h2><?php _e( 'Pushover Notification for WordPress Settings', CKPN_CORE_TEXT_DOMAIN ); ?></h2>
			<?php if ( isset( $logs_cleared ) && $logs_cleared ) {
			?><div id="setting-error-settings_updated" class="updated settings-error"><p><strong><?php _e( 'Logs Cleared', CKPN_CORE_TEXT_DOMAIN ); ?></strong></p></div><?php
		}
		?>
			<form method="post" action="options.php">
				<?php wp_nonce_field( 'ckpn-update-options' ); ?>
				<table class="form-table">
					<tr valign="top">
						<th scope="row"><?php _e( 'Application API Token/Key', CKPN_CORE_TEXT_DOMAIN ); ?><br /><span style="font-size: x-small;"><?php _e( 'Available from within your Pushover Account', CKPN_CORE_TEXT_DOMAIN ); ?></span></th>
						<td>
						<input size="50" type="text" name="ckpn_pushover_notifications_settings[application_key]" placeholder="<?php _e( 'Enter Application Key', CKPN_CORE_TEXT_DOMAIN ); ?>" <?php if ( $current['application_key'] != '' ) {?>value="<?php echo htmlspecialchars( $current['application_key'] ); ?>"<?php ;}?> />
						<br />
						<?php _e( 'To Create an application visit <a href="https://pushover.net/apps" target="_blank">https://pushover.net/apps</a>', CKPN_CORE_TEXT_DOMAIN ); ?>
						</td>
					</tr>
					<tr valign="top">
						<th scope="row"><?php _e( 'Your User Key', CKPN_CORE_TEXT_DOMAIN ); ?><br /><span style="font-size: x-small;"><?php _e( 'Available from within your Pushover Account', CKPN_CORE_TEXT_DOMAIN ); ?></span></th>
						<td>
						<input size="50" type="text" name="ckpn_pushover_notifications_settings[api_key]" placeholder="<?php _e( 'Enter User Key', CKPN_CORE_TEXT_DOMAIN ); ?>" <?php if ( $current['api_key'] != '' ) {?>value="<?php echo htmlspecialchars( $current['api_key'] ); ?>"<?php ;}?> /><br />
						<?php printf( __( 'This is the Administrator User Key. Other users can add their keys by editing their <a href="%s/wp-admin/profile.php">Profile</a> under the \'Contact Info\' Section.<br />
						Adding user specific keys allows you to be notified of Password Reset requests and other user specific notifications.', CKPN_CORE_TEXT_DOMAIN ), get_bloginfo( 'url' ) ); ?>
						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e( 'Send Notifications For:', CKPN_CORE_TEXT_DOMAIN ); ?><br /><span style="font-size: x-small;"><?php _e( 'What do you want to be notified of?', CKPN_CORE_TEXT_DOMAIN ); ?></span></th>
						<td>
							<input type="checkbox" name="ckpn_pushover_notifications_settings[new_user]" value="1" <?php if ( $current['new_user'] == '1' ) {?>checked="checked"<?php ;}?> /> <?php _e( 'New Users', CKPN_CORE_TEXT_DOMAIN ); ?><br />
							<input type="checkbox" name="ckpn_pushover_notifications_settings[new_comment]" value="1" <?php if ( $current['new_comment'] == '1' ) {?>checked="checked"<?php ;}?> /> <?php _e( 'New Comments', CKPN_CORE_TEXT_DOMAIN ); ?> <br />
							&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" name="ckpn_pushover_notifications_settings[notify_authors]" value="1" <?php if ( $current['notify_authors'] == '1' ) {?>checked="checked"<?php ;}?> /> <?php _e( 'Notify the Post Author (for muti-author blogs)', CKPN_CORE_TEXT_DOMAIN ); ?><br />
							<input type="checkbox" name="ckpn_pushover_notifications_settings[password_reset]" value="1" <?php if ( $current['password_reset'] == '1' ) {?>checked="checked"<?php ;}?> /> <?php _e( 'Notify users when password resets are requested for their accounts?', CKPN_CORE_TEXT_DOMAIN ); ?> <br />
						</td>
					</tr>

					<?php do_action( 'ckpn_notification_checkbox_filter' ); ?>

					<tr valign="top">
						<th scope="row"><?php _e( 'Advanced &amp; Debug Options:', CKPN_CORE_TEXT_DOMAIN ); ?><br /><span style="font-size: x-small;"><?php _e( 'With great power, comes great responsiblity.', CKPN_CORE_TEXT_DOMAIN ); ?></span></th>
						<td>
							<input type="checkbox" name="ckpn_pushover_notifications_settings[sslverify]" value="1" <?php if ($current['sslverify'] == '1' ) {?>checked="checked"<?php ;}?> /> <?php _e( 'Verify SSL from api.pushover.net?', CKPN_CORE_TEXT_DOMAIN ); ?><br />
							<small><?php _e( 'Disable if you get API Notification Errors in the logs. Caused when CURL cannot verify SSL Certificates', CKPN_CORE_TEXT_DOMAIN ); ?></small><br />
							<input type="checkbox" name="ckpn_pushover_notifications_settings[logging]" value="1" <?php if ($current['logging'] == '1' ) {?>checked="checked"<?php ;}?> /> <?php _e( 'Enable Logging', CKPN_CORE_TEXT_DOMAIN ); ?> - <small><a href="<?php echo wp_nonce_url( '?page=pushover-notifications&clear_logs=true', 'ckpn_clear_logs' ); ?>"><?php _e( 'Clear Logs', CKPN_CORE_TEXT_DOMAIN ); ?></a></small><br />
							<small><?php _e( 'Enable or Disable Logging', CKPN_CORE_TEXT_DOMAIN ); ?></small><br />
						</td>
					</tr>


					<input type="hidden" name="action" value="update" />
					<input type="hidden" name="page_options" value="ckpn_pushover_notifications_settings" />

					<?php settings_fields( 'ckpn-update-options' ); ?>
				</table>
				<input type="submit" class="button-primary" value="<?php _e( 'Save Changes' ) ?>" />
			</form>
			<?php
		if ( $current['logging'] == 1 ) {
		?>
			<h3><?php _e( 'Logs:', CKPN_CORE_TEXT_DOMAIN ); ?></h3>
<pre>
<?php // No indent to preserve formatting
			echo get_option( 'ckpn_logs' );
?>
</pre>
		<?php
		}
		?>
		</div>
		<?php
	}

	public function ckpn_add_contact_item( $contact_methods ) {
		$contact_methods['ckpn_user_key'] = 'Pushover User Key';

		return $contact_methods;
	}

	public function ckpn_register_settings() {
		register_setting( 'ckpn-update-options', 'ckpn_pushover_notifications_settings' );
	}

	/*
 	* ckpn_edd_missing_core_nag
 	*
 	* Warns a user if they don't have the core Pushover Notifications plugin.
 	*
 	*
 	* @return void
 	*/
	public function ckpn_edd_missing_nag() {
		if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) && 
			! is_plugin_active( 'pushover-notifications-edd-ext/pushover-notifications-edd-ext.php' ) &&
			( isset($_REQUEST['page'] ) && $_REQUEST['page'] == 'pushover-notifications' ) ) {
			printf( '<div class="error"> <p> %s </p> </div>', __( 'Get Pushover Notifications for your Easy Digital Downloads Sales with the <a href="https://easydigitaldownloads.com/extension/pushover-notifications/" target="_blank">Pushover Notifications Extension</a>.', CKPN_CORE_TEXT_DOMAIN ) );
		}
	}

	public function ckpn_loaddomain() {
		load_plugin_textdomain(CKPN_CORE_TEXT_DOMAIN, false, '/pushover-notifications/languages/' );
	}

	public function get_options() {
		return $this->getOptions();
	}

}

$ckpn_loaded = CKPushoverNotifications::getInstance();

